@@ -39,21 +39,22 @@ class HelpLayout(Layout):
             return dedent(
                 """
                 [white]
-                      k - [cyan]up[/cyan]
-                      j - [cyan]down[/cyan]
-                      g - [cyan]go to top[/cyan]
-                      G - [cyan]go to bottom[/cyan]
-                l Enter - [cyan]explore selected attribute[/cyan]
-                  h Esc - [cyan]go back to parent object[/cyan]
-                    [ ] - [cyan]switch attribute type (public/private)[/cyan]
-                    { } - [cyan]switch pane[/cyan]
-                      p - [cyan]toggle full preview[/cyan]
-                      d - [cyan]toggle full docstring[/cyan]
-                      f - [cyan]open fullscreen view[/cyan]
-                      H - [cyan]open help page on selected attribute[/cyan]
-                      r - [cyan]return the selected object[/cyan]
-                      ? - [cyan]toggle help page[/cyan]
-                    q Q - [cyan]quit[/cyan]
+                      k ↑ - [cyan]up[/cyan]
+                      j ↓ - [cyan]down[/cyan]
+                        g - [cyan]go to top[/cyan]
+                        G - [cyan]go to bottom[/cyan]
+                l → Enter - [cyan]explore selected attribute[/cyan]
+                  h ← Esc - [cyan]go back to parent object[/cyan]
+                      [ ] - [cyan]switch attribute type (public/private)[/cyan]
+                      { } - [cyan]switch pane[/cyan]
+                        p - [cyan]toggle full preview[/cyan]
+                        d - [cyan]toggle full docstring[/cyan]
+                        o - [cyan]toggle stack view[/cyan]
+                        f - [cyan]open fullscreen view[/cyan]
+                        H - [cyan]open help page on selected attribute[/cyan]
+                        r - [cyan]return the selected object[/cyan]
+                        ? - [cyan]toggle help page[/cyan]
+                      q Q - [cyan]quit[/cyan]
                 """
             ).strip()
 
